﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;

using Newtonsoft.Json;

namespace TestClient
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        static void Main()
        {
            // 포트 5569는 서버 설정과 일치시킨다.
            using(HttpClient client = GetHTTPClient("http://localhost:5569/"))
            {
                List<string> valueList;

                Console.WriteLine();
                Console.WriteLine("값 리스트를 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                valueList = GetValueList(client).GetAwaiter().GetResult();

                for(int i = 0; i < valueList.Count; i++)
                {
                    Console.WriteLine(valueList[i]);
                }


                Console.WriteLine();
                Console.WriteLine("값을 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                string value = GetValue(client, 0).GetAwaiter().GetResult();

                Console.WriteLine(value);


                Console.WriteLine();
                Console.WriteLine("값을 추가합니다.");
                Console.WriteLine("--------------------------------------------------");

                bool status = AddValue(client, "JAVA").GetAwaiter().GetResult();

                Console.WriteLine(status);


                Console.WriteLine();
                Console.WriteLine("값 리스트를 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                valueList = GetValueList(client).GetAwaiter().GetResult();

                for(int i = 0; i < valueList.Count; i++)
                {
                    Console.WriteLine(valueList[i]);
                }


                Console.WriteLine();
                Console.WriteLine("값을 수정합니다.");
                Console.WriteLine("--------------------------------------------------");

                status = UpdateValue(client, 3, "JAVA2").GetAwaiter().GetResult();

                Console.WriteLine(status);


                Console.WriteLine();
                Console.WriteLine("값 리스트를 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                valueList = GetValueList(client).GetAwaiter().GetResult();

                for(int i = 0; i < valueList.Count; i++)
                {
                    Console.WriteLine(valueList[i]);
                }


                Console.WriteLine();
                Console.WriteLine("값을 삭제합니다.");
                Console.WriteLine("--------------------------------------------------");

                status = DeleteValue(client, 3).GetAwaiter().GetResult();

                Console.WriteLine(status);


                Console.WriteLine();
                Console.WriteLine("값 리스트를 조회합니다.");
                Console.WriteLine("--------------------------------------------------");

                valueList = GetValueList(client).GetAwaiter().GetResult();

                for(int i = 0; i < valueList.Count; i++)
                {
                    Console.WriteLine(valueList[i]);
                }


                Console.WriteLine("--------------------------------------------------");

                Console.WriteLine("아무 키나 눌러 주시기 바랍니다.");

                Console.ReadKey(true);
            }
        }

        #endregion

        #region HTTP 클라이언트 구하기 - GetHTTPClient(baseAddress)

        /// <summary>
        /// HTTP 클라이언트 구하기
        /// </summary>
        /// <param name="baseAddress">기준 주소</param>
        /// <returns>HTTP 클라이언트</returns>
        private static HttpClient GetHTTPClient(string baseAddress)
        {
            HttpClient client = new HttpClient();

            client.BaseAddress = new Uri(baseAddress);

            client.DefaultRequestHeaders.Accept.Clear();

            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            return client;
        }

        #endregion

        #region 값 리스트 구하기 - GetValueList(client)

        /// <summary>
        /// 값 리스트 구하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <returns>값 리스트</returns>
        private static async Task<List<string>> GetValueList(HttpClient client)
        {
            string url = string.Format("api/value/");

            HttpResponseMessage response = await client.GetAsync(url);

            if(response.IsSuccessStatusCode)
            {
                TransferItem item = await response.Content.ReadAsAsync<TransferItem>();

                if(item.Status.Equals("OK"))
                {
                    return JsonConvert.DeserializeObject<List<string>>(item.Content);
                }
            }

            return null;
        }

        #endregion
        #region 값 구하기 - GetValue(client, index)

        /// <summary>
        /// 값 구하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="index">ID</param>
        /// <returns>값</returns>
        private static async Task<string> GetValue(HttpClient client, int index)
        {
            string url = string.Format("api/value/{0}", index);

            HttpResponseMessage response = await client.GetAsync(url);

            if(response.IsSuccessStatusCode)
            {
                TransferItem item = await response.Content.ReadAsAsync<TransferItem>();

                if(item.Status.Equals("OK"))
                {
                    return JsonConvert.DeserializeObject<string>(item.Content);
                }
            }

            return null;
        }

        #endregion

        #region 값 추가하기 - AddValue(client, value)

        /// <summary>
        /// 값 추가하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="value">값</param>
        /// <returns>처리 결과</returns>
        private static async Task<bool> AddValue(HttpClient client, string value)
        {
            string url = string.Format("api/value/");

            var response = await client.PostAsJsonAsync<string>(url, value);

            if(response.IsSuccessStatusCode)
            {
                TransferItem item = await response.Content.ReadAsAsync<TransferItem>();

                if(item.Status.Equals("OK"))
                {
                    return true;
                }
            }

            return false;
        }

        #endregion
        #region 값 수정하기 - UpdateValue(client, index, value)

        /// <summary>
        /// 값 수정하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="index">인덱스</param>
        /// <param name="value">값</param>
        /// <returns>처리 결과</returns>
        private static async Task<bool> UpdateValue(HttpClient client, int index, string value)
        {
            string url = string.Format("api/value/{0}", index);

            var response = await client.PutAsJsonAsync(url, value);

            if(response.IsSuccessStatusCode)
            {
                TransferItem item = await response.Content.ReadAsAsync<TransferItem>();

                if(item.Status.Equals("OK"))
                {
                    return true;
                }
            }

            return false;
        }

        #endregion
        #region 값 삭제하기 - DeleteValue(client, index)

        /// <summary>
        /// 값 삭제하기
        /// </summary>
        /// <param name="client">클라이언트</param>
        /// <param name="index">인덱스</param>
        /// <returns>처리 결과</returns>
        private static async Task<bool> DeleteValue(HttpClient client, int index)
        {
            string url = string.Format("api/value/{0}", index);

            var response = await client.DeleteAsync(url);

            if(response.IsSuccessStatusCode)
            {
                TransferItem item = await response.Content.ReadAsAsync<TransferItem>();

                if(item.Status.Equals("OK"))
                {
                    return true;
                }
            }

            return false;
        }

        #endregion
    }
}