﻿using Microsoft.AspNetCore.Mvc;

using Newtonsoft.Json;

using TestServer.Models;

namespace TestServer.Controllers
{
    /// <summary>
    /// 값 컨트롤러
    /// </summary>
    [ApiController]
    [Route("api/[controller]")]
    public class ValueController : ControllerBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 추가하기 - AddValue(value)

        /// <summary>
        /// 값 추가하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>전송 항목</returns>
        [HttpPost]
        public TransferItem AddValue([FromBody]string value)
        {
            TransferItem item = new TransferItem();

            if(ValueRepository.ValueList.Contains(value))
            {
                item.Status  = "FAILURE";
                item.Content = null;

                return item;
            }
            else
            {
                ValueRepository.ValueList.Add(value);

                item.Status  = "OK";
                item.Content = null;

                return item;
            }
        }

        #endregion
        #region 값 수정하기 - UpdateValue(index, value)

        /// <summary>
        /// 값 수정하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <param name="value">값</param>
        /// <returns>전송 항목</returns>
        [HttpPut("{index:int}")]
        public TransferItem UpdateValue(int index, [FromBody]string value)
        {
            TransferItem item = new TransferItem();

            if(index < 0 || index >= ValueRepository.ValueList.Count)
            {
                item.Status  = "FAILURE";
                item.Content = null;

                return item;
            }

            ValueRepository.ValueList[index] = value;

            item.Status  = "OK";
            item.Content = null;

            return item;
        }

        #endregion
        #region 값 삭제하기 - DeleteValue(index)

        /// <summary>
        /// 값 삭제하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>전송 항목</returns>
        [HttpDelete("{index:int}")]
        public TransferItem DeleteValue(int index)
        {
            TransferItem item = new TransferItem();

            if(index < 0 || index >= ValueRepository.ValueList.Count)
            {
                item.Status  = "FAILURE";
                item.Content = null;

                return item;
            }

            ValueRepository.ValueList.RemoveAt(index);

            item.Status  = "OK";
            item.Content = null;

            return item;
        }

        #endregion

        #region 값 리스트 구하기 - GetValueList()

        /// <summary>
        /// 값 리스트 구하기
        /// </summary>
        /// <returns>전송 항목</returns>
        [HttpGet]
        public TransferItem GetValueList()
        {
            TransferItem item = new TransferItem();

            item.Status  = "OK";
            item.Content = JsonConvert.SerializeObject(ValueRepository.ValueList);

            return item;
        }

        #endregion
        #region 값 구하기 - GetValue(index)

        /// <summary>
        /// 값 구하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>전송 항목</returns>
        [HttpGet("{index:int}")]
        public TransferItem GetValue(int index)
        {
            TransferItem item = new TransferItem();

            if(index < 0 || index >= ValueRepository.ValueList.Count)
            {
                item.Status = "FAILURE";
                item.Content   = null;

                return item;
            }

            item.Status  = "OK";
            item.Content = JsonConvert.SerializeObject(ValueRepository.ValueList[index]);

            return item;
        }

        #endregion
    }
}